<?php
namespace FiloBlu\Flow\Model\Connector;
/**
 * Created by PhpStorm.
 * User: marcovalenziano
 * Date: 07/05/15
 * Time: 12:40
 */

class Ftp extends AbstractConnector{

    private $host;
    private $username;
    private $password;
    private $passive = 0;
    public $type = "FTP";
    public $ftpDir;
    public $incomingDir;
    public $localDir;
    /**
    * @var Monolog\Logger
    */
    protected $_logger;
    
    
    /*
     * Init FTP Class retrieving configuration
     */
    public function __construct(
            \FiloBlu\Flow\Helper\LoggerProvider $_loggerProvider
    ){
        parent::__construct($_loggerProvider);
        $this->_logger = $_loggerProvider->getLogger();
    }

    
    public function setConfig($config)
    {
        $this->_config = $config;

        $this->host = (String) @$this->_config['host'];
        $this->username = (String) @$this->_config['username'];
        $this->password = (String) @$this->_config['password'];
        $this->passive= (String)@$this->_config['passive'];
        $this->ftpDir= (String) @$this->_config['ftp_dir'];
        $this->incomingDir= (String) @$this->_config['incomingDir'];
        $this->localDir= BP . "/var/" .(String) $this->_config['local_dir'];
        $this->connection=null;

        $this->_mask = (String) $this->_config['regex'];

        if(!file_exists($this->localDir)){
            mkdir($this->localDir);
        }

    }
    

    public function getLocalDir() {

        return $this->localDir;

    }

    public function openConnection() {

        $host = $this->host;

        if (!$this->connection) {
            if(!$host) {
                $this->_logger->info(__METHOD__.' -> Missing HOST');
                throw new \Exception('Missing HOST');
            }
                

            $this->connection = ftp_connect($host);
            if (!$this->connection) {
                $this->_logger->info(__METHOD__.' -> FTP connection failed');
                throw new \Exception('FTP connection failed');
            }
            $this->login($this->username,$this->password);
        }
        return $this->connection;
    }

    
    public function getConnection() {

        $this->openConnection();

        return $this->connection;

    }


    public function login($user=null, $pass=null) {

        $connection = $this->connection;

        $login_result = ftp_login($connection, $user, $pass);
        if (!$login_result) {
            $this->_logger->info(__METHOD__.' -> FTP login failed');
            throw new \Exception('FTP login failed');
        }

        //Modalità passiva attivata
        ftp_pasv($connection, $this->passive);
    }

    /**
     *
     * @param type $connection Connection
     * @param string $dir Directory to list
     * @return type List of file names
     */
    public function listDir($dir){
        
        $this->_logger->info(__METHOD__);

        $connection = $this->getConnection();

        ftp_pasv($connection, true);

        putenv('TMPDIR='.getenv('TMPDIR'));

        if (ftp_chdir($connection,$dir)) {
            $list = ftp_nlist($connection, ".");
        }
        else {
            $list = array();
        }

        return $list;
    }

    /**
     * Scarica un file dall'FTP
     * @param type $connection
     * @param string $local_file
     * @param string $server_file
     * @throws \Exception
     */
    public function downloadFile($local_file, $server_file, $mode = FTP_ASCII) {
        
        $this->_logger->info(__METHOD__);

        $connection = $this->getConnection();
        
        try{
            ftp_chdir($connection,$this->ftpDir);
        }
        catch (\Exception $e){
            $this->_logger->info($e->getMessage() . ' -> ' . $e->getCode());
        }
        
        try{
            $res = ftp_get($connection, $local_file, $server_file, $mode);
        }
        catch (\Exception $e){
            $this->_logger->info($e->getMessage() . ' -> ' . $e->getCode());
        }

        if(!$res) {
            $this->_logger->info(__METHOD__ . "FTP HELPER: download failed {$server_file} > {$local_file}");
            throw new \Exception("FTP HELPER: download failed {$server_file} > {$local_file}");
        }
        else {
            chmod ( $local_file , 0777 );
            return true;
        }
    }

    public function uploadFile($remote_file, $xmlPath,$mode = FTP_ASCII) {
        
        $connection = $this->getConnection();

        $this->_logger->info("Upload del file {$remote_file}");

        $res = ftp_put($connection, $remote_file, $xmlPath, $mode); //FTP_ASCII per i file di testo in generale

        if(!$res) {
            $this->_logger->info("FTP HELPER: upload failed {$remote_file} > {$xmlPath}");
            throw new \Exception("FTP HELPER: upload failed {$remote_file} > {$xmlPath}");
        }
        else {
            $this->_logger->info("FTP HELPER: Uploaded {$remote_file} > {$xmlPath}");
        }
        return true;
    }

    /**
     *
     * @param string $serverFilename
     * @return bool
     */
    public function deleteFile($serverFilename) {
        
        $this->_logger->info(__METHOD__);

        $connection = $this->getConnection();

        ftp_chdir($connection,$this->ftpDir);

        return ftp_delete($connection, $serverFilename);
    }


    public function close() {
        $connection = $this->getConnection();
        return ftp_close($connection);
    }
    
    /*
     * List, Download and Delete resources from ftp
     */
    public function getResources(){
        
        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection();
        $this->login($this->username,$this->password);

        $resources = $this->listDir($this->ftpDir);
        $downloadedFiles = array();

        foreach($resources as $resource){
            $basefile = basename($resource);
            $localFile = $this->localDir."/".$basefile;

            // la regex deve avere caratteri delimiter! nel campo backend
             if (preg_match($this->_mask,$basefile)) {

                $downloadedFiles[] = $basefile;

            }

        }

        $this->close();
        return $downloadedFiles;
    }

    public function receiveResource($resource, $copy = null, $ftp_mode = FTP_ASCII, $keep_conn_open = false, $fix_filename = false) {

        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection($this->host);
        $this->login($this->username,$this->password);

        $downloadedFile = array();

        if(!is_dir($this->localDir) && trim($this->localDir) != '') {
            mkdir($this->localDir, 0777, true);
        }

            $basefile = basename($resource);
            if ($fix_filename) {
                $basefile = $this->fixFilename($basefile);
            }            
            $localFile = $this->localDir."/".$basefile;

            if (file_exists($localFile)) {
                $deleted = $this->deleteFile($resource);
                $this->_logger->info('FTP - file '.$basefile.' already exists');
                return false;
            }

            try {
                if ($this->downloadFile($localFile, $resource, $ftp_mode)) {
                    if($this->deleteFile($resource)) {
                        $downloadedFile = $basefile;
                        if($copy){
                            $fileInfo = pathinfo($localFile);
                            $fileCopyName = $copy.'.'.$fileInfo['extension'];
                            $fileCopy = $this->localDir."/".$fileCopyName;
                            if(copy($localFile,$fileCopy)){
                                $downloadedFile = array('basefile'=>$downloadedFile,'copy'=>$fileCopyName);
                            }
                        }
                    }
                }
            } catch(\Exception $e) {
                $this->_logger->info($e->getMessage());
            }


            if (!$keep_conn_open) {
                $this->close();
            }

            return $downloadedFile;

    }

    public function isReceived() {

    }

    public function sendResource($resource){
        
        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection($this->host);
        $this->login($this->username,$this->password);

        $downloadedFiles = array();

        if(!is_dir($this->localDir) && trim($this->localDir) != '') {
            mkdir($this->localDir, 0777, true);
        }

        $basefile = basename($resource);
        $localFile = $this->localDir."/".$basefile;

        if (file_exists($localFile)) {
            if ($this->uploadFile($localFile, $resource)) {
                unlink($localFile);
                return true;
            }
        }
        return false;
    }

    public function setFtpDir( $dir ){
        $this->ftpDir = $dir;
        return $this;
    }

    public function setLocalDir( $dir ){
        $this->localDir = $dir;
        return $this;
    }

    public function fixFilename($fileName)
    {
        $fileName = preg_replace('/[^a-z0-9_\\-\\.]+/i', '', $fileName);
        return $fileName;
    }    
}
