<?php

namespace FiloBlu\Flow\Model\From;

class Stock extends AbstractFrom {
    
    /**
    * @var Monolog\Logger
    */
    protected $_logger;
    protected $_helper_flow;
    protected $_product_factory;
    protected $_stock_registry;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        array $data = []
    ) {
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        $this->_product_factory = $productFactory;
        $this->_stock_registry = $stockRegistry;
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    } 

     public function process(){
         
        //  $this->_logger->info(__METHOD__);
         
        $product_id = $this->_product_factory->create()->getIdBySku($this->getSku());  
         
        //$product = $this->_product_factory->create()->setStoreId(0);
          
        if(!$product_id){
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(true, $message);
        }
       
        if($product_id){ 
            
            // salvo il prezzo sul prodotto semplice
            //$product->load($product_id);
            
            $stock = $this->_stock_registry->getStockItem($product_id);
               
            $_in_stock = false;
            
            //$this->_logger->info('Process STOCK ---> ' . $product_id);
      
            try {
               
                $qty = intval($this->getData('fisico_disponibile'));
                $stock->setQty($qty);
                
                if ($qty > $stock->getMinQty()){
                    $_in_stock = true;
                    $stock->setQty($qty);
                    $stock->setIsInStock($_in_stock);
                }
                elseif($stock->getBackorders() != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO && $stock->getMinQty() == 0) {
                    $_in_stock = true;
                    $stock->setQty($qty);
                    $stock->setIsInStock($_in_stock);
                }
                else {
                    $stock->setQty($qty);
                    $stock->setIsInStock($_in_stock);
                }

                $stock->save();
                
                $this->_logger->info('Process STOCK - qty ' . $qty . ' ID: '.$product_id);

                if($_in_stock) {
                    
                     // se il prodotto semplice ha qty > 0, il configurabile padre deve tornare in stock
                    $parent_product_id = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable')->getParentIdsByChild($product_id);
                    if(isset($parent_product_id[0])){
                       
                        $_parent_stock = $this->_stock_registry->getStockItem($parent_product_id[0]);
                        $_parent_stock->setIsInStock($_in_stock);
                        $_parent_stock->save();
                    }
                }
                

            } catch(\Exception $e){
                 $this->_logger->info('Errore nel salvataggio dello Stock. ' . $e->getMessage() );
            }
           
        }

        return $this; 
    }

   


}